"""! @file main.py
@brief Main file for the application
@author xlanro00
"""

# Import basic libraries
import argparse as ap
import sys
import json
from datetime import datetime

# Libraries for image processing
import numpy as np
import matplotlib.pyplot as plt
from PIL import Image
import cv2 as cv

# Import custom image filter library
import filters as flt

class apply_filters:
    def __init__(self):
        # Parse arguments from command line
        self.parse_arguments()
        self.input_file = self.args.input_file
        self.output_file = self.args.output_file
        self.dpi = self.args.dpi
        self.filters = self.args.filters
        self.mirror = self.args.mirror if self.args.mirror else 0

        # Parse configuration from json file
        if self.args.config:
            self.config_file = self.args.config[0]
            self.preset_name = self.args.config[1]
            self.config = json.load(open(self.config_file))
            self.parse_conf()
            # If no preset name given, create one from time
            #self.preset_name = "preset_" + datetime.now().strftime("%d_%m_%Y_%H_%M_%S")

        # If no config file given, expect filters in command line
        else:
            self.filters = self.args.filters

        # Convert dimensions
        self.img = Image.open(self.input_file)
        if self.img is None:
            sys.exit("Could not load the fingerprint.")
        #self.convert_dpi()
        #self.resize_image()

        #convert to numpy array for further processing
        self.img = np.array(self.img)

        if self.mirror:
            self.mirror_image()

        # Apply all filters
        self.apply_filter()


    def parse_conf(self):

        # Parse configuration file if given.
        try:
            self.filters = self.config[self.preset_name]
        except(KeyError):
            print("Preset not found", file=sys.stderr)


    def parse_arguments(self):

        # Parse arguments
        parser = ap.ArgumentParser(prog = 'main.py', description = 
            'Program for processing a 2D image into 3D fingerprint.')

        # positional arguments
        parser.add_argument("input_file", type = str, help = "Location with input file")
        parser.add_argument("output_file", type = str, help = "Output file location")
        parser.add_argument("dpi", type = int, help = "Scanner dpi")

        # boolean switch
        parser.add_argument('-m', "--mirror", help = "Mirror input image",
                            type = bool, action = ap.BooleanOptionalAction)

        # file with configuration containing presets, new preset name
        # pair argument - give both or none
        parser.add_argument('--config', nargs=2, metavar=('config_file', 'preset'),
                            help='Config file with presets, name of the preset')

        # array of unknown length, all filter names saved inside
        parser.add_argument('filters', type = str, nargs = '*', help = "List of filter names")

        self.args = parser.parse_args()


    def filter_factory(self, filter_name):
        # selects filter method of filters library
        # better this than a 100 if/else
        return getattr(flt, filter_name)


    def convert_dpi(self):

        # conversion from inches to milimeters
        self.size = np.empty(2)
        self.size[0] = self.img.size[0]  # / self.dpi * 25.4 # width
        self.size[1] = self.img.size[1]  # / self.dpi * 25.4 # height


    def resize_image(self):

        # open image as python image object
        print("Resize image", file = sys.stderr)

        self.convert_dpi()
        #self.img = self.img.resize((np.array(self.size)).astype(int))


    def mirror_image(self):

        # mirror image when mirroring is needed
        # should be used only if we want a positive form
        print("Mirroring image", file=sys.stderr)
        self.img = cv.flip(self.img, 1)  # 1 for vertical mirror


    def apply_filter(self):
        
        if len(self.filters) == 0:
            # save original image
            filter = flt.filter_none
            filter.apply(self)
        else:
            for filter_name in self.filters:
                filter = self.filter_factory(filter_name)
                filter.apply(self)
        self.save_image()


    def print_size(self, size):
        print("Width: " + str(size[0]), file = sys.stderr)
        print("Height: " + str(size[1]), file = sys.stderr)


    def save_image(self):
        # Save processed image.
        plt.xticks([]), plt.yticks([])
        plt.axis('off')
        print("Saving image", file = sys.stderr)

        # TODO idk what dpi means, and if it should be put in here
        plt.savefig(self.output_file)#, dpi=self.dpi)


app = apply_filters()
